@@ -34,7 +34,9 @@ enum
     PROP_TEXT,
     PROP_URI,
     PROP_ICON,
-    PROP_TOKEN
+    PROP_TOKEN,
+    PROP_ADDED,
+    PROP_VISITS
 };
 
 static void
@@ -69,8 +71,8 @@ katze_item_class_init (KatzeItemClass* class)
                                      PROP_NAME,
                                      g_param_spec_string (
                                      "name",
-                                     _("Name"),
-                                     _("The name of the item"),
+                                     "Name",
+                                     "The name of the item",
                                      NULL,
                                      flags));
 
@@ -78,8 +80,8 @@ katze_item_class_init (KatzeItemClass* class)
                                      PROP_TEXT,
                                      g_param_spec_string (
                                      "text",
-                                     _("Text"),
-                                     _("The descriptive text of the item"),
+                                     "Text",
+                                     "The descriptive text of the item",
                                      NULL,
                                      flags));
 
@@ -87,8 +89,8 @@ katze_item_class_init (KatzeItemClass* class)
                                      PROP_URI,
                                      g_param_spec_string (
                                      "uri",
-                                     _("URI"),
-                                     _("The URI of the item"),
+                                     "URI",
+                                     "The URI of the item",
                                      NULL,
                                      flags));
 
@@ -96,8 +98,8 @@ katze_item_class_init (KatzeItemClass* class)
                                      PROP_ICON,
                                      g_param_spec_string (
                                      "icon",
-                                     _("Icon"),
-                                     _("The icon of the item"),
+                                     "Icon",
+                                     "The icon of the item",
                                      NULL,
                                      flags));
 
@@ -105,10 +107,33 @@ katze_item_class_init (KatzeItemClass* class)
                                      PROP_TOKEN,
                                      g_param_spec_string (
                                      "token",
-                                     _("Token"),
-                                     _("The token of the item"),
+                                     "Token",
+                                     "The token of the item",
                                      NULL,
                                      flags));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_ADDED,
+                                     g_param_spec_int64 (
+                                     "added",
+                                     "Added",
+                                     "When the item was added",
+                                     G_MININT64,
+                                     G_MAXINT64,
+                                     0,
+                                     flags));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_VISITS,
+                                     g_param_spec_int (
+                                     "visits",
+                                     "Visits",
+                                     "The number of visits of the item",
+                                     G_MININT,
+                                     G_MAXINT,
+                                     0,
+                                     flags));
+
 }
 
 
@@ -144,19 +169,25 @@ katze_item_set_property (GObject*      object,
     switch (prop_id)
     {
     case PROP_NAME:
-        item->name = g_value_dup_string (value);
+        katze_assign (item->name, g_value_dup_string (value));
         break;
     case PROP_TEXT:
-        item->text = g_value_dup_string (value);
+        katze_assign (item->text, g_value_dup_string (value));
         break;
     case PROP_URI:
-        item->uri = g_value_dup_string (value);
+        katze_assign (item->uri, g_value_dup_string (value));
         break;
     case PROP_ICON:
-        item->icon = g_value_dup_string (value);
+        katze_assign (item->icon, g_value_dup_string (value));
         break;
     case PROP_TOKEN:
-        item->token = g_value_dup_string (value);
+        katze_assign (item->token, g_value_dup_string (value));
+        break;
+    case PROP_ADDED:
+        item->added = g_value_get_int64 (value);
+        break;
+    case PROP_VISITS:
+        item->visits = g_value_get_int (value);
         break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -189,6 +220,12 @@ katze_item_get_property (GObject*    object,
     case PROP_TOKEN:
         g_value_set_string (value, item->token);
         break;
+    case PROP_ADDED:
+        g_value_set_int64 (value, item->added);
+        break;
+    case PROP_VISITS:
+        g_value_set_int (value, item->visits);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -374,3 +411,108 @@ katze_item_set_token (KatzeItem*   item,
     katze_assign (item->token, g_strdup (token));
     g_object_notify (G_OBJECT (item), "token");
 }
+
+/**
+ * katze_item_get_added:
+ * @item: a #KatzeItem
+ *
+ * Determines when @item was added.
+ *
+ * Return value: a timestamp
+ **/
+gint64
+katze_item_get_added (KatzeItem* item)
+{
+    g_return_val_if_fail (KATZE_IS_ITEM (item), 0);
+
+    return item->added;
+}
+
+/**
+ * katze_item_set_added:
+ * @item: a #KatzeItem
+ * @added: a timestamp
+ *
+ * Sets when @item was added.
+ **/
+void
+katze_item_set_added (KatzeItem* item,
+                      gint64     added)
+{
+    g_return_if_fail (KATZE_IS_ITEM (item));
+
+    item->added = added;
+    g_object_notify (G_OBJECT (item), "added");
+}
+
+/**
+ * katze_item_get_visits:
+ * @item: a #KatzeItem
+ *
+ * Retrieves the number of visits of @item.
+ *
+ * Return value: the number of visits
+ **/
+gint
+katze_item_get_visits (KatzeItem* item)
+{
+    g_return_val_if_fail (KATZE_IS_ITEM (item), -1);
+
+    return item->visits;
+}
+
+/**
+ * katze_item_set_visits:
+ * @item: a #KatzeItem
+ * @visits: an integer
+ *
+ * Sets the number of visits of @item.
+ **/
+void
+katze_item_set_visits (KatzeItem* item,
+                       gint       visits)
+{
+    g_return_if_fail (KATZE_IS_ITEM (item));
+
+    item->visits = visits;
+    g_object_notify (G_OBJECT (item), "visits");
+}
+
+/**
+ * katze_item_get_parent:
+ * @item: a #KatzeItem
+ *
+ * Determines the parent of @item.
+ *
+ * Return value: the parent of the item
+ **/
+gpointer
+katze_item_get_parent (KatzeItem* item)
+{
+    g_return_val_if_fail (KATZE_IS_ITEM (item), NULL);
+
+    return item->parent;
+}
+
+/**
+ * katze_item_set_parent:
+ * @item: a #KatzeItem
+ * @parent: the new parent
+ *
+ * Sets the parent of @item.
+ *
+ * This is intended for item container implementations and
+ * should not be used otherwise.
+ **/
+void
+katze_item_set_parent (KatzeItem* item,
+                       gpointer   parent)
+{
+    g_return_if_fail (KATZE_IS_ITEM (item));
+
+    if (parent)
+        g_object_ref (parent);
+    katze_object_assign (item->parent, parent);
+    /* g_object_notify (G_OBJECT (item), "parent"); */
+}
+
