@@ -41,6 +41,10 @@ struct _KatzeItem
     gchar* uri;
     gchar* icon;
     gchar* token;
+    gint64 added;
+    gint   visits;
+
+    KatzeItem* parent;
 };
 
 struct _KatzeItemClass
@@ -89,6 +93,27 @@ void
 katze_item_set_token              (KatzeItem*      item,
                                    const gchar*    token);
 
+gint64
+katze_item_get_added              (KatzeItem*      item);
+
+void
+katze_item_set_added              (KatzeItem*      item,
+                                   gint64          added);
+
+gint
+katze_item_get_visits             (KatzeItem*      item);
+
+void
+katze_item_set_visits             (KatzeItem*      item,
+                                   gint            visits);
+
+gpointer
+katze_item_get_parent             (KatzeItem*      item);
+
+void
+katze_item_set_parent             (KatzeItem*      item,
+                                   gpointer        parent);
+
 G_END_DECLS
 
 #endif /* __MIDORI_WEB_ITEM_H__ */
