@@ -17,6 +17,6 @@
 #include "katze-item.h"
 #include "katze-list.h"
 #include "katze-array.h"
-#include "katze-xbel.h"
+#include "katze-arrayaction.h"
 
 #endif /* __KATZE_H__ */
