@@ -32,8 +32,10 @@ struct _MidoriApp
     GtkAccelGroup* accel_group;
 
     MidoriWebSettings* settings;
+    KatzeArray* bookmarks;
     KatzeArray* trash;
     KatzeArray* search_engines;
+    KatzeArray* history;
 
     gpointer instance;
 };
@@ -47,10 +49,12 @@ enum
     PROP_0,
 
     PROP_SETTINGS,
+    PROP_BOOKMARKS,
     PROP_TRASH,
+    PROP_SEARCH_ENGINES,
     PROP_BROWSER,
-    PROP_BROWSER_COUNT,
-    PROP_SEARCH_ENGINES
+    PROP_HISTORY,
+    PROP_BROWSER_COUNT
 };
 
 enum {
@@ -85,6 +89,8 @@ midori_app_get_property (GObject*    object,
 static void
 midori_app_class_init (MidoriAppClass* class)
 {
+    GObjectClass* gobject_class;
+
     signals[ADD_BROWSER] = g_signal_new (
         "add-browser",
         G_TYPE_FROM_CLASS (class),
@@ -106,31 +112,48 @@ midori_app_class_init (MidoriAppClass* class)
         g_cclosure_marshal_VOID__VOID,
         G_TYPE_NONE, 0);
 
-    GObjectClass* gobject_class = G_OBJECT_CLASS (class);
+    gobject_class = G_OBJECT_CLASS (class);
     gobject_class->constructor = midori_app_constructor;
     gobject_class->finalize = midori_app_finalize;
     gobject_class->set_property = midori_app_set_property;
     gobject_class->get_property = midori_app_get_property;
 
-    MidoriAppClass* midoriapp_class = MIDORI_APP_CLASS (class);
-    midoriapp_class->add_browser = midori_app_add_browser;
-    midoriapp_class->quit = midori_app_quit;
+    class->add_browser = midori_app_add_browser;
+    class->quit = midori_app_quit;
 
     g_object_class_install_property (gobject_class,
                                      PROP_SETTINGS,
                                      g_param_spec_object (
                                      "settings",
-                                     _("Settings"),
-                                     _("The associated settings"),
+                                     "Settings",
+                                     "The associated settings",
                                      MIDORI_TYPE_WEB_SETTINGS,
                                      G_PARAM_READWRITE));
 
+    g_object_class_install_property (gobject_class,
+                                     PROP_BOOKMARKS,
+                                     g_param_spec_object (
+                                     "bookmarks",
+                                     "Bookmarks",
+                                     "The bookmarks folder, containing all bookmarks",
+                                     KATZE_TYPE_ARRAY,
+                                     G_PARAM_READWRITE));
+
     g_object_class_install_property (gobject_class,
                                      PROP_TRASH,
                                      g_param_spec_object (
                                      "trash",
-                                     _("Trash"),
-                                     _("The trash, collecting recently closed tabs and windows"),
+                                     "Trash",
+                                     "The trash, collecting recently closed tabs and windows",
+                                     KATZE_TYPE_ARRAY,
+                                     G_PARAM_READWRITE));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_SEARCH_ENGINES,
+                                     g_param_spec_object (
+                                     "search-engines",
+                                     "Search Engines",
+                                     "The list of search engines",
                                      KATZE_TYPE_ARRAY,
                                      G_PARAM_READWRITE));
 
@@ -138,8 +161,8 @@ midori_app_class_init (MidoriAppClass* class)
                                      PROP_BROWSER,
                                      g_param_spec_object (
                                      "browser",
-                                     _("Browser"),
-                                     _("The current browser"),
+                                     "Browser",
+                                     "The current browser",
                                      MIDORI_TYPE_BROWSER,
                                      G_PARAM_READABLE));
 
@@ -147,19 +170,21 @@ midori_app_class_init (MidoriAppClass* class)
                                      PROP_BROWSER_COUNT,
                                      g_param_spec_uint (
                                      "browser-count",
-                                     _("Browser Count"),
-                                     _("The current number of browsers"),
+                                     "Browser Count",
+                                     "The current number of browsers",
                                      0, G_MAXUINT, 0,
                                      G_PARAM_READABLE));
 
     g_object_class_install_property (gobject_class,
-                                     PROP_SEARCH_ENGINES,
+                                     PROP_HISTORY,
                                      g_param_spec_object (
-                                     "search-engines",
-                                     _("Search Engines"),
-                                     _("The list of search engines"),
+                                     "history",
+                                     "History",
+                                     "The list of history items",
                                      KATZE_TYPE_ARRAY,
                                      G_PARAM_READWRITE));
+
+
 }
 
 static GObject*
@@ -183,6 +208,7 @@ midori_browser_message_received_cb (UniqueApp*         instance,
                                     MidoriApp*         app)
 {
   UniqueResponse response;
+  MidoriBrowser* browser;
   gchar** uris;
 
   switch (command)
@@ -193,7 +219,26 @@ midori_browser_message_received_cb (UniqueApp*         instance,
       gtk_window_present (GTK_WINDOW (app->browser));
       response = UNIQUE_RESPONSE_OK;
       break;
+  case UNIQUE_NEW:
+      browser = g_object_new (MIDORI_TYPE_BROWSER,
+                              "settings", app->settings,
+                              "bookmarks", app->bookmarks,
+                              "trash", app->trash,
+                              "search-engines", app->search_engines,
+                              "history", app->history,
+                              NULL);
+      /* FIXME: Should open the homepage according to settings */
+      midori_browser_add_uri (browser, "");
+      midori_app_add_browser (app, browser);
+      gtk_window_set_screen (GTK_WINDOW (app->browser),
+                             unique_message_data_get_screen (message));
+      gtk_widget_show (GTK_WIDGET (browser));
+      response = UNIQUE_RESPONSE_OK;
+      break;
   case UNIQUE_OPEN:
+      gtk_window_set_screen (GTK_WINDOW (app->browser),
+                             unique_message_data_get_screen (message));
+      gtk_window_present (GTK_WINDOW (app->browser));
       uris = unique_message_data_get_uris (message);
       if (!uris)
           response = UNIQUE_RESPONSE_FAIL;
@@ -201,7 +246,8 @@ midori_browser_message_received_cb (UniqueApp*         instance,
       {
           while (*uris)
           {
-              midori_browser_add_uri (app->browser, *uris);
+              midori_browser_set_current_page (app->browser,
+                  midori_browser_add_uri (app->browser, *uris));
               uris++;
           }
           /* g_strfreev (uris); */
@@ -233,8 +279,10 @@ midori_app_init (MidoriApp* app)
     app->accel_group = gtk_accel_group_new ();
 
     app->settings = midori_web_settings_new ();
-    app->trash = katze_array_new (KATZE_TYPE_XBEL_ITEM);
-    app->search_engines = katze_array_new (KATZE_TYPE_ITEM);
+    app->bookmarks = NULL;
+    app->trash = NULL;
+    app->search_engines = NULL;
+    app->history = NULL;
 
     #if HAVE_UNIQUE
     display_name = g_strdup (gdk_display_get_name (gdk_display_get_default ()));
@@ -261,8 +309,16 @@ midori_app_finalize (GObject* object)
     g_list_free (app->browsers);
     g_object_unref (app->accel_group);
 
-    g_object_unref (app->settings);
-    g_object_unref (app->trash);
+    if (app->settings)
+        g_object_unref (app->settings);
+    if (app->bookmarks)
+        g_object_unref (app->bookmarks);
+    if (app->trash)
+        g_object_unref (app->trash);
+    if (app->search_engines)
+        g_object_unref (app->search_engines);
+    if (app->history)
+        g_object_unref (app->history);
 
     if (app->instance)
         g_object_unref (app->instance);
@@ -285,6 +341,11 @@ midori_app_set_property (GObject*      object,
         g_object_ref (app->settings);
         /* FIXME: Propagate settings to all browsers */
         break;
+    case PROP_BOOKMARKS:
+        katze_object_assign (app->bookmarks, g_value_get_object (value));
+        g_object_ref (app->bookmarks);
+        /* FIXME: Propagate bookmarks to all browsers */
+        break;
     case PROP_TRASH:
         katze_object_assign (app->trash, g_value_get_object (value));
         g_object_ref (app->trash);
@@ -295,6 +356,10 @@ midori_app_set_property (GObject*      object,
         g_object_ref (app->search_engines);
         /* FIXME: Propagate search engines to all browsers */
         break;
+    case PROP_HISTORY:
+        katze_object_assign (app->history, g_value_get_object (value));
+        /* FIXME: Propagate history to all browsers */
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -314,18 +379,24 @@ midori_app_get_property (GObject*    object,
     case PROP_SETTINGS:
         g_value_set_object (value, app->settings);
         break;
+    case PROP_BOOKMARKS:
+        g_value_set_object (value, app->bookmarks);
+        break;
     case PROP_TRASH:
         g_value_set_object (value, app->trash);
         break;
+    case PROP_SEARCH_ENGINES:
+        g_value_set_object (value, app->search_engines);
+        break;
+    case PROP_HISTORY:
+        g_value_set_object (value, app->history);
+        break;
     case PROP_BROWSER:
         g_value_set_object (value, app->browser);
         break;
     case PROP_BROWSER_COUNT:
         g_value_set_uint (value, g_list_length (app->browsers));
         break;
-    case PROP_SEARCH_ENGINES:
-        g_value_set_object (value, app->search_engines);
-        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -338,6 +409,7 @@ midori_browser_focus_in_event_cb (MidoriBrowser* browser,
                                   MidoriApp*     app)
 {
     app->browser = browser;
+    g_object_notify (G_OBJECT (app), "browser");
     return FALSE;
 }
 
@@ -348,8 +420,10 @@ midori_browser_new_window_cb (MidoriBrowser* browser,
 {
     MidoriBrowser* new_browser = g_object_new (MIDORI_TYPE_BROWSER,
                                                "settings", app->settings,
+                                               "bookmarks", app->bookmarks,
                                                "trash", app->trash,
                                                "search-engines", app->search_engines,
+                                               "history", app->history,
                                                NULL);
     midori_browser_add_uri (new_browser, uri);
     gtk_widget_show (GTK_WIDGET (new_browser));
@@ -454,6 +528,33 @@ midori_app_instance_send_activate (MidoriApp* app)
     return FALSE;
 }
 
+/**
+ * midori_app_instance_send_new_browser:
+ * @app: a #MidoriApp
+ *
+ * Sends a message to an instance of Midori already
+ * running on the default display, asking to open a new browser.
+ *
+ * Return value: %TRUE if the message was sent successfully
+ **/
+gboolean
+midori_app_instance_send_new_browser (MidoriApp* app)
+{
+    #if HAVE_UNIQUE
+    UniqueResponse response;
+    #endif
+
+    g_return_val_if_fail (MIDORI_IS_APP (app), FALSE);
+    g_return_val_if_fail (midori_app_instance_is_running (app), FALSE);
+
+    #if HAVE_UNIQUE
+    response = unique_app_send_message (app->instance, UNIQUE_NEW, NULL);
+    if (response == UNIQUE_RESPONSE_OK)
+        return TRUE;
+    #endif
+    return FALSE;
+}
+
 /**
  * midori_app_instance_send_uris:
  * @app: a #MidoriApp
