@@ -56,10 +56,10 @@ struct _MidoriBrowserClass
 
     void
     (*add_tab)                 (MidoriBrowser*       browser,
-                                GtkWidget*           widget);
+                                GtkWidget*           view);
     void
     (*remove_tab)              (MidoriBrowser*       browser,
-                                GtkWidget*           widget);
+                                GtkWidget*           view);
     void
     (*activate_action)         (MidoriBrowser*       browser,
                                 const gchar*         name);
@@ -82,8 +82,8 @@ midori_browser_remove_tab             (MidoriBrowser*     browser,
                                        GtkWidget*         widget);
 
 gint
-midori_browser_add_xbel_item          (MidoriBrowser*     browser,
-                                       KatzeXbelItem*     xbel_item);
+midori_browser_add_item               (MidoriBrowser*     browser,
+                                       KatzeItem*         item);
 
 gint
 midori_browser_add_uri                (MidoriBrowser*     browser,
@@ -110,11 +110,8 @@ midori_browser_set_current_tab        (MidoriBrowser*     browser,
 GtkWidget*
 midori_browser_get_current_tab        (MidoriBrowser*     browser);
 
-GtkWidget*
-midori_browser_get_current_web_view   (MidoriBrowser*     browser);
-
-KatzeXbelItem*
-midori_browser_get_proxy_xbel_folder  (MidoriBrowser*     browser);
+KatzeArray*
+midori_browser_get_proxy_array        (MidoriBrowser*     browser);
 
 void
 midori_browser_quit                   (MidoriBrowser*     browser);
