@@ -11,12 +11,10 @@
 
 #include "midori-locationaction.h"
 
-#include "main.h"
 #include "gtkiconentry.h"
 
-#include <katze/katze.h>
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtk.h>
 
 struct _MidoriLocationAction
 {
@@ -181,8 +179,8 @@ midori_location_action_class_init (MidoriLocationActionClass* class)
                                      PROP_PROGRESS,
                                      g_param_spec_double (
                                      "progress",
-                                     _("Progress"),
-                                     _("The current progress of the action"),
+                                     "Progress",
+                                     "The current progress of the action",
                                      0.0, 1.0, 0.0,
                                      G_PARAM_WRITABLE));
 
@@ -190,8 +188,8 @@ midori_location_action_class_init (MidoriLocationActionClass* class)
                                      PROP_SECONDARY_ICON,
                                      g_param_spec_string (
                                      "secondary-icon",
-                                     _("Secondary"),
-                                     _("The stock ID of the secondary icon"),
+                                     "Secondary",
+                                     "The stock ID of the secondary icon",
                                      NULL,
                                      G_PARAM_WRITABLE));
 }
@@ -289,7 +287,7 @@ midori_location_action_create_tool_item (GtkAction* action)
     gtk_container_add (GTK_CONTAINER (toolitem), alignment);
     gtk_widget_show (alignment);
 
-    return GTK_WIDGET (toolitem);
+    return toolitem;
 }
 
 static void
@@ -387,6 +385,7 @@ static void
 midori_location_action_disconnect_proxy (GtkAction* action,
                                          GtkWidget* proxy)
 {
+    /* FIXME: This is wrong */
     g_signal_handlers_disconnect_by_func (proxy,
         G_CALLBACK (gtk_action_activate), action);
 
