@@ -11,6 +11,7 @@
 
 #include "midori-panel.h"
 
+#include "midori-view.h"
 #include "sokoke.h"
 #include "compat.h"
 
@@ -143,8 +144,8 @@ midori_panel_class_init (MidoriPanelClass* class)
                                      PROP_SHADOW_TYPE,
                                      g_param_spec_enum (
                                      "shadow-type",
-                                     _("Shadow Type"),
-                                     _("Appearance of the shadow around each panel"),
+                                     "Shadow Type",
+                                     "Appearance of the shadow around each panel",
                                      GTK_TYPE_SHADOW_TYPE,
                                      GTK_SHADOW_NONE,
                                      flags));
@@ -153,8 +154,8 @@ midori_panel_class_init (MidoriPanelClass* class)
                                      PROP_MENU,
                                      g_param_spec_object (
                                      "menu",
-                                     _("Menu"),
-                                     _("Menu to hold panel items"),
+                                     "Menu",
+                                     "Menu to hold panel items",
                                      GTK_TYPE_MENU,
                                      G_PARAM_READWRITE));
 
@@ -162,8 +163,8 @@ midori_panel_class_init (MidoriPanelClass* class)
                                      PROP_PAGE,
                                      g_param_spec_int (
                                      "page",
-                                     _("Page"),
-                                     _("The index of the current page"),
+                                     "Page",
+                                     "The index of the current page",
                                      -1, G_MAXINT, -1,
                                      flags));
 }
@@ -373,22 +374,27 @@ midori_panel_append_page (MidoriPanel* panel,
     g_return_val_if_fail (stock_id != NULL, -1);
     g_return_val_if_fail (label != NULL, -1);
 
-    scrolled = gtk_scrolled_window_new (NULL, NULL);
-    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
-                                    GTK_POLICY_AUTOMATIC,
-                                    GTK_POLICY_AUTOMATIC);
-    GTK_WIDGET_SET_FLAGS (scrolled, GTK_CAN_FOCUS);
-    gtk_widget_show (scrolled);
-    gobject_class = G_OBJECT_GET_CLASS (child);
-    if (GTK_WIDGET_CLASS (gobject_class)->set_scroll_adjustments_signal)
-        widget = child;
+    if (GTK_IS_SCROLLED_WINDOW (child))
+        scrolled = child;
     else
     {
-        widget = gtk_viewport_new (NULL, NULL);
-        gtk_widget_show (widget);
-        gtk_container_add (GTK_CONTAINER (widget), child);
+        scrolled = gtk_scrolled_window_new (NULL, NULL);
+        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
+                                        GTK_POLICY_AUTOMATIC,
+                                        GTK_POLICY_AUTOMATIC);
+        GTK_WIDGET_SET_FLAGS (scrolled, GTK_CAN_FOCUS);
+        gtk_widget_show (scrolled);
+        gobject_class = G_OBJECT_GET_CLASS (child);
+        if (GTK_WIDGET_CLASS (gobject_class)->set_scroll_adjustments_signal)
+            widget = child;
+        else
+        {
+            widget = gtk_viewport_new (NULL, NULL);
+            gtk_widget_show (widget);
+            gtk_container_add (GTK_CONTAINER (widget), child);
+        }
+        gtk_container_add (GTK_CONTAINER (scrolled), widget);
     }
-    gtk_container_add (GTK_CONTAINER (scrolled), widget);
     gtk_container_add (GTK_CONTAINER (panel->notebook), scrolled);
 
     if (!toolbar)
@@ -396,7 +402,7 @@ midori_panel_append_page (MidoriPanel* panel,
     gtk_widget_show (toolbar);
     gtk_container_add (GTK_CONTAINER (panel->toolbook), toolbar);
 
-    n = midori_panel_page_num (panel, child);
+    n = midori_panel_page_num (panel, scrolled);
 
     g_object_set_data (G_OBJECT (child), "label", (gchar*)label);
 
@@ -449,7 +455,13 @@ static GtkWidget*
 _midori_panel_child_for_scrolled (MidoriPanel* panel,
                                   GtkWidget*   scrolled)
 {
-    GtkWidget* child = gtk_bin_get_child (GTK_BIN (scrolled));
+    GtkWidget* child;
+
+    /* This is a lazy hack, we should have a way of determining
+       whether the scrolled is the actual child. */
+    if (MIDORI_IS_VIEW (scrolled))
+        return scrolled;
+    child = gtk_bin_get_child (GTK_BIN (scrolled));
     if (GTK_IS_VIEWPORT (child))
         child = gtk_bin_get_child (GTK_BIN (child));
     return child;
@@ -469,9 +481,11 @@ GtkWidget*
 midori_panel_get_nth_page (MidoriPanel* panel,
                            guint        page_num)
 {
+    GtkWidget* scrolled;
+
     g_return_val_if_fail (MIDORI_IS_PANEL (panel), NULL);
 
-    GtkWidget* scrolled = gtk_notebook_get_nth_page (
+    scrolled = gtk_notebook_get_nth_page (
         GTK_NOTEBOOK (panel->notebook), page_num);
     if (scrolled)
         return _midori_panel_child_for_scrolled (panel, scrolled);
@@ -498,7 +512,13 @@ static GtkWidget*
 _midori_panel_scrolled_for_child (MidoriPanel* panel,
                                   GtkWidget*   child)
 {
-    GtkWidget* scrolled = gtk_widget_get_parent (GTK_WIDGET (child));
+    GtkWidget* scrolled;
+
+    /* This is a lazy hack, we should have a way of determining
+       whether the scrolled is the actual child. */
+    if (MIDORI_IS_VIEW (child))
+        return child;
+    scrolled = gtk_widget_get_parent (GTK_WIDGET (child));
     if (GTK_IS_VIEWPORT (scrolled))
         scrolled = gtk_widget_get_parent (scrolled);
     return scrolled;
