@@ -59,44 +59,45 @@ midori_preferences_class_init (MidoriPreferencesClass* class)
                                      g_param_spec_object (
                                      "settings",
                                      "Settings",
-                                     _("Settings instance to provide properties"),
+                                     "Settings instance to provide properties",
                                      MIDORI_TYPE_WEB_SETTINGS,
                                      G_PARAM_WRITABLE));
 }
 
 static void
-clear_button_clicked_cb (GtkWidget* button, GtkWidget* file_chooser)
+midori_preferences_response_cb (MidoriPreferences* preferences,
+                                gint               response)
 {
-    gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (file_chooser), "");
-    /* Emit signal manually for Gtk doesn't emit it otherwise */
-    #if GTK_CHECK_VERSION (2, 12, 0)
-    g_signal_emit_by_name (file_chooser, "file-set");
-    #else
-    g_signal_emit_by_name (file_chooser, "selection-changed");
-    #endif
+    if (response == GTK_RESPONSE_CLOSE)
+        gtk_widget_destroy (GTK_WIDGET (preferences));
 }
 
+#ifdef GDK_WINDOWING_QUARTZ
+    #define USE_OSX_STYLE
+#endif
+
 static void
 midori_preferences_init (MidoriPreferences* preferences)
 {
+    gchar* dialog_title;
+
     preferences->notebook = NULL;
 
-    gchar* dialog_title = g_strdup_printf (_("%s Preferences"),
-                                           g_get_application_name ());
+    dialog_title = g_strdup_printf (_("%s Preferences"),
+                                    g_get_application_name ());
     g_object_set (preferences,
                   "icon-name", GTK_STOCK_PREFERENCES,
                   "title", dialog_title,
                   "has-separator", FALSE,
                   NULL);
+    #ifndef USE_OSX_STYLE
     gtk_dialog_add_buttons (GTK_DIALOG (preferences),
         GTK_STOCK_HELP, GTK_RESPONSE_HELP,
         GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
         NULL);
-    /* TODO: Implement some kind of help function */
-    gtk_dialog_set_response_sensitive (GTK_DIALOG (preferences),
-                                       GTK_RESPONSE_HELP, FALSE);
+    #endif
     g_signal_connect (preferences, "response",
-                      G_CALLBACK (gtk_widget_destroy), preferences);
+                      G_CALLBACK (midori_preferences_response_cb), NULL);
 
     /* TODO: Do we want tooltips for explainations or can we omit that? */
     g_free (dialog_title);
@@ -204,6 +205,42 @@ proxy_download_manager_icon_cb (GtkWidget*     entry,
     return FALSE;
 }
 
+#ifdef USE_OSX_STYLE
+static void
+midori_preferences_help_clicked_cb (GtkWidget* button,
+                                    GtkDialog* dialog)
+{
+    gtk_dialog_response (dialog, GTK_RESPONSE_HELP);
+}
+
+static void
+midori_preferences_toolbutton_clicked_cb (GtkWidget* toolbutton,
+                                          GtkWidget* page)
+{
+    gpointer notebook = g_object_get_data (G_OBJECT (toolbutton), "notebook");
+    guint n = gtk_notebook_page_num (notebook, page);
+    gtk_notebook_set_current_page (notebook, n);
+}
+#endif
+
+static inline void
+midori_preferences_add_toolbutton (GtkWidget*   toolbar,
+                                   GtkWidget**  toolbutton,
+                                   const gchar* icon,
+                                   const gchar* label,
+                                   GtkWidget*   page)
+{
+#ifdef USE_OSX_STYLE
+    *toolbutton = GTK_WIDGET (*toolbutton ? gtk_radio_tool_button_new_from_widget (
+        GTK_RADIO_TOOL_BUTTON (*toolbutton)) : gtk_radio_tool_button_new (NULL));
+    gtk_tool_button_set_label (GTK_TOOL_BUTTON (*toolbutton), label);
+    gtk_tool_button_set_stock_id (GTK_TOOL_BUTTON (*toolbutton), icon);
+    gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (*toolbutton), -1);
+    g_signal_connect (*toolbutton, "clicked",
+        G_CALLBACK (midori_preferences_toolbutton_clicked_cb), page);
+#endif
+}
+
 /**
  * midori_preferences_set_settings:
  * @settings: the settings
@@ -217,6 +254,8 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
                                  MidoriWebSettings* settings)
 {
     GtkSizeGroup* sizegroup;
+    GtkWidget* toolbar;
+    GtkWidget* toolbutton;
     GtkWidget* page;
     GtkWidget* frame;
     GtkWidget* table;
@@ -234,17 +273,37 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
 
     preferences->notebook = gtk_notebook_new ();
     gtk_container_set_border_width (GTK_CONTAINER (preferences->notebook), 6);
-    sizegroup = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-    #define PAGE_NEW(__label) page = gtk_vbox_new (FALSE, 0); \
-     gtk_container_set_border_width (GTK_CONTAINER (page), 5); \
+
+    #ifdef USE_OSX_STYLE
+    gtk_notebook_set_show_tabs (GTK_NOTEBOOK (preferences->notebook), FALSE);
+    gtk_notebook_set_show_border (GTK_NOTEBOOK (preferences->notebook), FALSE);
+    toolbar = gtk_toolbar_new ();
+    gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_BOTH);
+    gtk_toolbar_set_show_arrow (GTK_TOOLBAR (toolbar), FALSE);
+    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (preferences)->vbox),
+                        toolbar, FALSE, FALSE, 0);
+    #endif
+    toolbar = NULL;
+    toolbutton = NULL;
+
+    sizegroup = NULL;
+    #define PAGE_NEW(__icon, __label) \
+     page = gtk_vbox_new (FALSE, 0); \
+     midori_preferences_add_toolbutton (toolbar, &toolbutton, \
+         __icon, __label, page); \
+     if (toolbutton) g_object_set_data (G_OBJECT (toolbutton), \
+      "notebook", preferences->notebook); \
+     if (sizegroup) g_object_unref (sizegroup); \
+     sizegroup = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL); \
+     gtk_container_set_border_width (GTK_CONTAINER (page), 4); \
      gtk_notebook_append_page (GTK_NOTEBOOK (preferences->notebook), page, \
                                gtk_label_new (__label))
     #define FRAME_NEW(__label) frame = sokoke_hig_frame_new (__label); \
-     gtk_container_set_border_width (GTK_CONTAINER (frame), 5); \
+     gtk_container_set_border_width (GTK_CONTAINER (frame), 4); \
      gtk_box_pack_start (GTK_BOX (page), frame, FALSE, FALSE, 0);
     #define TABLE_NEW(__rows, __cols) table = gtk_table_new ( \
                                        __rows, __cols, FALSE); \
-     gtk_container_set_border_width (GTK_CONTAINER (table), 5); \
+     gtk_container_set_border_width (GTK_CONTAINER (table), 4); \
      gtk_container_add (GTK_CONTAINER (frame), table);
     #define WIDGET_ADD(__widget, __left, __right, __top, __bottom) \
      gtk_table_attach (GTK_TABLE (table), __widget \
@@ -264,7 +323,7 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
      gtk_container_add (GTK_CONTAINER (align), __widget); \
      FILLED_ADD (align, __left, __right, __top, __bottom)
     /* Page "General" */
-    PAGE_NEW (_("General"));
+    PAGE_NEW (GTK_STOCK_HOME, _("General"));
     FRAME_NEW (_("Startup"));
     TABLE_NEW (2, 2);
     label = katze_property_label (settings, "load-on-startup");
@@ -298,7 +357,7 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     FILLED_ADD (hbox, 1, 2, 1, 2);
 
     /* Page "Appearance" */
-    PAGE_NEW (_("Appearance"));
+    PAGE_NEW (GTK_STOCK_SELECT_FONT, _("Appearance"));
     FRAME_NEW (_("Font settings"));
     TABLE_NEW (5, 2);
     label = katze_property_label (settings, "default-font-family");
@@ -319,7 +378,7 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     FILLED_ADD (button, 1, 2, 2, 3);
 
     /* Page "Behavior" */
-    PAGE_NEW (_("Behavior"));
+    PAGE_NEW (GTK_STOCK_SELECT_COLOR, _("Behavior"));
     FRAME_NEW (_("Features"));
     TABLE_NEW (6, 2);
     button = katze_property_proxy (settings, "auto-load-images", NULL);
@@ -333,58 +392,32 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     button = katze_property_proxy (settings, "enable-scripts", NULL);
     INDENTED_ADD (button, 0, 1, 2, 3);
     button = katze_property_proxy (settings, "enable-plugins", NULL);
-    SPANNED_ADD(button, 1, 2, 2, 3);
-    label = katze_property_label (settings, "user-stylesheet-uri");
-    INDENTED_ADD (label, 0, 1, 3, 4);
-    hbox = gtk_hbox_new (FALSE, 4);
-    entry = katze_property_proxy (settings, "user-stylesheet-uri", "uri");
-    gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
-    button = gtk_button_new ();
-    gtk_container_add (GTK_CONTAINER (button),
-        gtk_image_new_from_stock (GTK_STOCK_CLEAR, GTK_ICON_SIZE_MENU));
-    g_signal_connect (button, "clicked",
-                      G_CALLBACK (clear_button_clicked_cb), entry);
-    gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 4);
-    FILLED_ADD (hbox, 1, 2, 3, 4);
-    if (g_object_class_find_property (G_OBJECT_GET_CLASS (settings), "zoom-step"))
-    {
-        label = katze_property_label (settings, "zoom-step");
-        INDENTED_ADD (label, 0, 1, 4, 5);
-        hbox = gtk_hbox_new (FALSE, 4);
-        entry = katze_property_proxy (settings, "zoom-step", NULL);
-        gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
-        FILLED_ADD (hbox, 1, 2, 4, 5);
-    }
-    else
-    {
-        label = gtk_label_new ("Zoom Stepping Value");
-        INDENTED_ADD (label, 0, 1, 4, 5);
-        hbox = gtk_hbox_new (FALSE, 4);
-        entry = gtk_label_new ("Not available in this WebKit version");
-        gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
-        FILLED_ADD (hbox, 1, 2, 4, 5);
-    }
+    SPANNED_ADD (button, 1, 2, 2, 3);
     label = katze_property_label (settings, "location-entry-search");
-    INDENTED_ADD (label, 0, 1, 5, 6);
+    INDENTED_ADD (label, 0, 1, 3, 4);
     entry = katze_property_proxy (settings, "location-entry-search", NULL);
-    FILLED_ADD (entry, 1, 2, 5, 6);
+    FILLED_ADD (entry, 1, 2, 3, 4);
 
     /* Page "Interface" */
-    PAGE_NEW (_("Interface"));
+    PAGE_NEW (GTK_STOCK_CONVERT, _("Interface"));
     FRAME_NEW (_("Navigationbar"));
-    TABLE_NEW (3, 2);
+    TABLE_NEW (4, 2);
     INDENTED_ADD (katze_property_label (settings, "toolbar-style"), 0, 1, 0, 1);
     button = katze_property_proxy (settings, "toolbar-style", NULL);
-    FILLED_ADD(button, 1, 2, 0, 1);
-    button = katze_property_proxy (settings, "show-new-tab", NULL);
-    INDENTED_ADD (button, 0, 1, 1, 2);
-    button = katze_property_proxy (settings, "show-web-search", NULL);
-    SPANNED_ADD (button, 1, 2, 1, 2);
-    button = katze_property_proxy (settings, "show-homepage", NULL);
-    INDENTED_ADD (button, 0, 1, 2, 3);
-    button = katze_property_proxy (settings, "show-trash", NULL);
-    SPANNED_ADD (button, 1, 2, 2, 3);
-    FRAME_NEW(_("Browsing"));
+    FILLED_ADD (button, 1, 2, 0, 1);
+    INDENTED_ADD (katze_property_label (settings, "toolbar-items"), 0, 1, 1, 2);
+    button = katze_property_proxy (settings, "toolbar-items", NULL);
+    FILLED_ADD (button, 1, 2, 1, 2);
+    button = gtk_label_new (NULL);
+    gtk_label_set_markup (GTK_LABEL (button),
+        "<span size='smaller'>"
+        "TabNew, Open, Print, Find, Location, Preferences, Window, Bookmarks\n"
+        "ReloadStop, ZoomIn, ZoomOut, Back, Forward, Homepage, Search, Trash"
+        "</span>");
+    FILLED_ADD (button, 0, 2, 2, 3);
+    button = katze_property_proxy (settings, "always-show-tabbar", NULL);
+    INDENTED_ADD (button, 0, 1, 3, 4);
+    FRAME_NEW (_("Browsing"));
     TABLE_NEW (3, 2);
     /* label = katze_property_label (settings, "open-new-pages-in");
     INDENTED_ADD (label, 0, 1, 0, 1);
@@ -393,16 +426,16 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     button = katze_property_proxy (settings, "middle-click-opens-selection", NULL);
     INDENTED_ADD (button, 0, 1, 1, 2);
     button = katze_property_proxy (settings, "open-tabs-in-the-background", NULL);
-    SPANNED_ADD (button, 1, 2, 1, 2);
+    WIDGET_ADD (button, 1, 2, 1, 2);
     /* button = katze_property_proxy (settings, "open-popups-in-tabs", NULL);
     SPANNED_ADD (button, 0, 1, 2, 3);*/
     button = katze_property_proxy (settings, "open-tabs-next-to-current", NULL);
-    SPANNED_ADD (button, 0, 1, 2, 3);
+    WIDGET_ADD (button, 0, 1, 2, 3);
     button = katze_property_proxy (settings, "close-buttons-on-tabs", NULL);
-    SPANNED_ADD (button, 1, 2, 2, 3);
+    WIDGET_ADD (button, 1, 2, 2, 3);
 
     /* Page "Network" */
-    PAGE_NEW (_("Network"));
+    /*PAGE_NEW (GTK_STOCK_NETWORK, _("Network"));
     FRAME_NEW (_("Network"));
     TABLE_NEW (2, 2);
     label = katze_property_label (settings, "http-proxy");
@@ -416,11 +449,11 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_("MB")),
                         FALSE, FALSE, 0);
-    FILLED_ADD (hbox, 1, 2, 1, 2);
+    FILLED_ADD (hbox, 1, 2, 1, 2);*/
 
     /* Page "Privacy" */
-    /* PAGE_NEW (_("Privacy"));
-    FRAME_NEW (_("Web Cookies"));
+    PAGE_NEW (GTK_STOCK_INDEX, _("Privacy"));
+    /*FRAME_NEW (_("Web Cookies"));
     TABLE_NEW (3, 2);
     label = katze_property_label (settings, "accept-cookies");
     INDENTED_ADD (label, 0, 1, 0, 1);
@@ -435,7 +468,7 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_("days")),
                         FALSE, FALSE, 0);
-    FILLED_ADD (hbox, 1, 2, 2, 3);
+    FILLED_ADD (hbox, 1, 2, 2, 3);*/
     FRAME_NEW (_("History"));
     TABLE_NEW (3, 2);
     button = katze_property_proxy (settings, "remember-last-visited-pages", NULL);
@@ -449,9 +482,23 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     button = katze_property_proxy (settings, "remember-last-form-inputs", NULL);
     SPANNED_ADD (button, 0, 2, 1, 2);
     button = katze_property_proxy (settings, "remember-last-downloaded-files", NULL);
-    SPANNED_ADD (button, 0, 2, 2, 3); */
+    SPANNED_ADD (button, 0, 2, 2, 3);
 
+    g_object_unref (sizegroup);
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG (preferences)->vbox),
                         preferences->notebook, FALSE, FALSE, 4);
+    #ifdef USE_OSX_STYLE
+    GtkWidget* icon;
+    hbox = gtk_hbox_new (FALSE, 0);
+    button = gtk_button_new ();
+    icon = gtk_image_new_from_stock (GTK_STOCK_HELP, GTK_ICON_SIZE_BUTTON);
+    gtk_button_set_image (GTK_BUTTON (button), icon);
+    g_signal_connect (button, "clicked",
+        G_CALLBACK (midori_preferences_help_clicked_cb), preferences);
+    gtk_box_pack_end (GTK_BOX (hbox),
+        button, FALSE, FALSE, 4);
+    gtk_box_pack_end (GTK_BOX (GTK_DIALOG (preferences)->vbox),
+        hbox, FALSE, FALSE, 0);
+    #endif
     gtk_widget_show_all (GTK_DIALOG (preferences)->vbox);
 }
