@@ -28,12 +28,15 @@ struct _MidoriWebSettings
     gint last_web_search;
     gchar* last_pageholder_uri;
 
+    gboolean show_menubar;
     gboolean show_navigationbar;
     gboolean show_bookmarkbar;
     gboolean show_panel;
     gboolean show_statusbar;
 
     MidoriToolbarStyle toolbar_style;
+    gchar* toolbar_items;
+    gboolean always_show_tabbar;
     gboolean show_new_tab;
     gboolean show_homepage;
     gboolean show_web_search;
@@ -46,7 +49,6 @@ struct _MidoriWebSettings
     gchar* location_entry_search;
     MidoriPreferredEncoding preferred_encoding;
 
-    gint tab_label_size;
     gboolean close_buttons_on_tabs;
     MidoriNewPage open_new_pages_in;
     gboolean middle_click_opens_selection;
@@ -81,12 +83,15 @@ enum
     PROP_LAST_WEB_SEARCH,
     PROP_LAST_PAGEHOLDER_URI,
 
+    PROP_SHOW_MENUBAR,
     PROP_SHOW_NAVIGATIONBAR,
     PROP_SHOW_BOOKMARKBAR,
     PROP_SHOW_PANEL,
     PROP_SHOW_STATUSBAR,
 
     PROP_TOOLBAR_STYLE,
+    PROP_TOOLBAR_ITEMS,
+    PROP_ALWAYS_SHOW_TABBAR,
     PROP_SHOW_NEW_TAB,
     PROP_SHOW_HOMEPAGE,
     PROP_SHOW_WEB_SEARCH,
@@ -99,7 +104,6 @@ enum
     PROP_LOCATION_ENTRY_SEARCH,
     PROP_PREFERRED_ENCODING,
 
-    PROP_TAB_LABEL_SIZE,
     PROP_CLOSE_BUTTONS_ON_TABS,
     PROP_OPEN_NEW_PAGES_IN,
     PROP_MIDDLE_CLICK_OPENS_SELECTION,
@@ -299,6 +303,14 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      flags));
 
 
+    g_object_class_install_property (gobject_class,
+                                     PROP_SHOW_MENUBAR,
+                                     g_param_spec_boolean (
+                                     "show-menubar",
+                                     _("Show Menubar"),
+                                     _("Whether to show the menubar"),
+                                     TRUE,
+                                     flags));
 
     g_object_class_install_property (gobject_class,
                                      PROP_SHOW_NAVIGATIONBAR,
@@ -347,6 +359,24 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      MIDORI_TOOLBAR_DEFAULT,
                                      flags));
 
+    g_object_class_install_property (gobject_class,
+                                     PROP_TOOLBAR_ITEMS,
+                                     g_param_spec_string (
+                                     "toolbar-items",
+                                     _("Toolbar Items"),
+                                     _("The items to show on the toolbar"),
+                                     "Back,Forward,ReloadStop,Location,Trash",
+                                     flags));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_ALWAYS_SHOW_TABBAR,
+                                     g_param_spec_boolean (
+                                     "always-show-tabbar",
+                                     _("Always Show Tabbar"),
+                                     _("Always show the tabbar"),
+                                     FALSE,
+                                     flags));
+
     g_object_class_install_property (gobject_class,
                                      PROP_SHOW_NEW_TAB,
                                      g_param_spec_boolean (
@@ -442,16 +472,6 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      flags));
 
 
-
-    g_object_class_install_property (gobject_class,
-                                     PROP_TAB_LABEL_SIZE,
-                                     g_param_spec_int (
-                                     "tab-label-size",
-                                     _("Tab Label Size"),
-                                     _("The desired tab label size"),
-                                     0, G_MAXINT, 10,
-                                     flags));
-
     g_object_class_install_property (gobject_class,
                                      PROP_CLOSE_BUTTONS_ON_TABS,
                                      g_param_spec_boolean (
@@ -546,7 +566,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      _("Remember last visited pages"),
                                      _("Whether the last visited pages are saved"),
                                      TRUE,
-                                     G_PARAM_READABLE));
+                                     flags));
 
     g_object_class_install_property (gobject_class,
                                      PROP_MAXIMUM_HISTORY_AGE,
@@ -555,7 +575,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      _("Maximum history age"),
                                      _("The maximum number of days to save the history for"),
                                      0, G_MAXINT, 30,
-                                     G_PARAM_READABLE));
+                                     flags));
 
     g_object_class_install_property (gobject_class,
                                      PROP_REMEMBER_LAST_FORM_INPUTS,
@@ -583,8 +603,8 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      "http-proxy",
                                      _("HTTP Proxy"),
                                      _("The proxy used for HTTP connections"),
-                                     g_getenv ("http_proxy"),
-                                     flags));
+                                     NULL,
+                                     G_PARAM_READABLE));
 
     g_object_class_install_property (gobject_class,
                                      PROP_CACHE_SIZE,
@@ -597,7 +617,8 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
 }
 
 static void
-notify_default_encoding_cb (GObject* object, GParamSpec* pspec)
+notify_default_encoding_cb (GObject*    object,
+                            GParamSpec* pspec)
 {
     MidoriWebSettings* web_settings = MIDORI_WEB_SETTINGS (object);
 
@@ -622,6 +643,8 @@ notify_default_encoding_cb (GObject* object, GParamSpec* pspec)
 static void
 midori_web_settings_init (MidoriWebSettings* web_settings)
 {
+    web_settings->http_proxy = NULL;
+
     g_signal_connect (web_settings, "notify::default-encoding",
                       G_CALLBACK (notify_default_encoding_cb), NULL);
 }
@@ -664,6 +687,9 @@ midori_web_settings_set_property (GObject*      object,
         katze_assign (web_settings->last_pageholder_uri, g_value_dup_string (value));
         break;
 
+    case PROP_SHOW_MENUBAR:
+        web_settings->show_menubar = g_value_get_boolean (value);
+        break;
     case PROP_SHOW_NAVIGATIONBAR:
         web_settings->show_navigationbar = g_value_get_boolean (value);
         break;
@@ -680,6 +706,12 @@ midori_web_settings_set_property (GObject*      object,
     case PROP_TOOLBAR_STYLE:
         web_settings->toolbar_style = g_value_get_enum (value);
         break;
+    case PROP_TOOLBAR_ITEMS:
+        katze_assign (web_settings->toolbar_items, g_value_dup_string (value));
+        break;
+    case PROP_ALWAYS_SHOW_TABBAR:
+        web_settings->always_show_tabbar = g_value_get_boolean (value);
+        break;
     case PROP_SHOW_NEW_TAB:
         web_settings->show_new_tab = g_value_get_boolean (value);
         break;
@@ -732,9 +764,6 @@ midori_web_settings_set_property (GObject*      object,
         }
         break;
 
-    case PROP_TAB_LABEL_SIZE:
-        web_settings->tab_label_size = g_value_get_int (value);
-        break;
     case PROP_CLOSE_BUTTONS_ON_TABS:
         web_settings->close_buttons_on_tabs = g_value_get_boolean (value);
         break;
@@ -779,7 +808,6 @@ midori_web_settings_set_property (GObject*      object,
 
     case PROP_HTTP_PROXY:
         katze_assign (web_settings->http_proxy, g_value_dup_string (value));
-        g_setenv ("http_proxy", web_settings->http_proxy ? web_settings->http_proxy : "", TRUE);
         break;
     case PROP_CACHE_SIZE:
         web_settings->cache_size = g_value_get_int (value);
@@ -822,6 +850,9 @@ midori_web_settings_get_property (GObject*    object,
         g_value_set_string (value, web_settings->last_pageholder_uri);
         break;
 
+    case PROP_SHOW_MENUBAR:
+        g_value_set_boolean (value, web_settings->show_menubar);
+        break;
     case PROP_SHOW_NAVIGATIONBAR:
         g_value_set_boolean (value, web_settings->show_navigationbar);
         break;
@@ -838,6 +869,12 @@ midori_web_settings_get_property (GObject*    object,
     case PROP_TOOLBAR_STYLE:
         g_value_set_enum (value, web_settings->toolbar_style);
         break;
+    case PROP_TOOLBAR_ITEMS:
+        g_value_set_string (value, web_settings->toolbar_items);
+        break;
+    case PROP_ALWAYS_SHOW_TABBAR:
+        g_value_set_boolean (value, web_settings->always_show_tabbar);
+        break;
     case PROP_SHOW_NEW_TAB:
         g_value_set_boolean (value, web_settings->show_new_tab);
         break;
@@ -870,9 +907,6 @@ midori_web_settings_get_property (GObject*    object,
         g_value_set_enum (value, web_settings->preferred_encoding);
         break;
 
-    case PROP_TAB_LABEL_SIZE:
-        g_value_set_int (value, web_settings->tab_label_size);
-        break;
     case PROP_CLOSE_BUTTONS_ON_TABS:
         g_value_set_boolean (value, web_settings->close_buttons_on_tabs);
         break;
@@ -944,54 +978,3 @@ midori_web_settings_new (void)
 
     return web_settings;
 }
-
-/**
- * midori_web_settings_copy:
- *
- * Copies an existing #MidoriWebSettings instance.
- *
- * Return value: a new #MidoriWebSettings
- **/
-MidoriWebSettings*
-midori_web_settings_copy (MidoriWebSettings* web_settings)
-{
-    g_return_val_if_fail (MIDORI_IS_WEB_SETTINGS (web_settings), NULL);
-
-    MidoriWebSettings* copy;
-    copy = MIDORI_WEB_SETTINGS (webkit_web_settings_copy (
-        WEBKIT_WEB_SETTINGS (web_settings)));
-    g_object_set (copy,
-                  "load-on-startup", web_settings->load_on_startup,
-                  "homepage", web_settings->homepage,
-                  "download-folder", web_settings->download_folder,
-                  "download-manager", web_settings->download_manager,
-                  "location-entry-search", web_settings->location_entry_search,
-                  "preferred-encoding", web_settings->preferred_encoding,
-
-                  "toolbar-style", web_settings->toolbar_style,
-                  "show-web-search", web_settings->show_web_search,
-                  "show-new-tab", web_settings->show_new_tab,
-                  "show-trash", web_settings->show_trash,
-
-                  "tab-label-size", web_settings->tab_label_size,
-                  "close-buttons-on-tabs", web_settings->close_buttons_on_tabs,
-                  "open-new-pages-in", web_settings->open_new_pages_in,
-                  "middle-click-opens-selection", web_settings->middle_click_opens_selection,
-                  "open-tabs-in-the-background", web_settings->open_tabs_in_the_background,
-                  "open-popups-in-tabs", web_settings->open_popups_in_tabs,
-
-                  "accept-cookies", web_settings->accept_cookies,
-                  "original-cookies-only", web_settings->original_cookies_only,
-                  "maximum-cookie-age", web_settings->maximum_cookie_age,
-
-                  "remember-last-visited-pages", web_settings->remember_last_visited_pages,
-                  "maximum-history-age", web_settings->maximum_history_age,
-                  "remember-last-form-inputs", web_settings->remember_last_form_inputs,
-                  "remember-last-downloaded-files", web_settings->remember_last_downloaded_files,
-
-                  "http-proxy", web_settings->http_proxy,
-                  "cache-size", web_settings->cache_size,
-                  NULL);
-
-    return copy;
-}
