@@ -14,7 +14,6 @@
 #endif
 
 #include "sokoke.h"
-#include "main.h"
 
 #if HAVE_UNISTD_H
     #include <unistd.h>
@@ -112,7 +111,8 @@ sokoke_magic_uri (const gchar* uri,
     if (!strstr (uri, "://"))
     {
         /* Do we have a domain, ip address or localhost? */
-        if (strchr (uri, '.') != NULL || !strcmp (uri, "localhost"))
+        if (strchr (uri, '.') != NULL || strchr (uri, ':')
+            || !strcmp (uri, "localhost"))
             return g_strconcat ("http://", uri, NULL);
         /* We don't want to search? So return early. */
         if (!search_engines)
@@ -192,87 +192,13 @@ sokoke_container_show_children (GtkContainer* container)
     gtk_container_foreach (container, (GtkCallback)(gtk_widget_show_all), NULL);
 }
 
-typedef struct
-{
-     GtkWidget* widget;
-     SokokeMenuPos position;
-} SokokePopupInfo;
-
-static void
-sokoke_widget_popup_position_menu (GtkMenu*  menu,
-                                   gint*     x,
-                                   gint*     y,
-                                   gboolean* push_in,
-                                   gpointer  user_data)
-{
-    gint wx, wy;
-    gint menu_width;
-    GtkRequisition menu_req;
-    GtkRequisition widget_req;
-    SokokePopupInfo* info = user_data;
-    GtkWidget* widget = info->widget;
-
-    /* Retrieve size and position of both widget and menu */
-    if (GTK_WIDGET_NO_WINDOW (widget))
-    {
-        gdk_window_get_position (widget->window, &wx, &wy);
-        wx += widget->allocation.x;
-        wy += widget->allocation.y;
-    }
-    else
-        gdk_window_get_origin (widget->window, &wx, &wy);
-    gtk_widget_size_request (GTK_WIDGET (menu), &menu_req);
-    gtk_widget_size_request (widget, &widget_req);
-    menu_width = menu_req.width;
-    gint widget_height = widget_req.height; /* Better than allocation.height */
-
-    /* Calculate menu position */
-    if (info->position == SOKOKE_MENU_POSITION_CURSOR)
-        ; /* Do nothing? */
-    else if (info->position == SOKOKE_MENU_POSITION_RIGHT)
-    {
-        *x = wx + widget->allocation.width - menu_width;
-        *y = wy + widget_height;
-    } else if (info->position == SOKOKE_MENU_POSITION_LEFT)
-    {
-        *x = wx;
-        *y = wy + widget_height;
-    }
-
-    *push_in = TRUE;
-}
-
-
 void
 sokoke_widget_popup (GtkWidget*      widget,
                      GtkMenu*        menu,
                      GdkEventButton* event,
                      SokokeMenuPos   pos)
 {
-    int button, event_time;
-    if (event)
-    {
-        button = event->button;
-        event_time = event->time;
-    }
-    else
-    {
-        button = 0;
-        event_time = gtk_get_current_event_time ();
-    }
-
-    if (!gtk_menu_get_attach_widget(menu))
-        gtk_menu_attach_to_widget (menu, widget, NULL);
-
-    if (widget)
-    {
-        SokokePopupInfo info = { widget, pos };
-        gtk_menu_popup (menu, NULL, NULL,
-                        sokoke_widget_popup_position_menu, &info,
-                        button, event_time);
-    }
-    else
-        gtk_menu_popup (menu, NULL, NULL, NULL, NULL, button, event_time);
+    katze_widget_popup (widget, menu, event, (KatzeMenuPos)pos);
 }
 
 typedef enum
@@ -285,6 +211,7 @@ typedef enum
 static SokokeDesktop
 sokoke_get_desktop (void)
 {
+    #ifdef GDK_WINDOWING_X11
     static SokokeDesktop desktop = SOKOKE_DESKTOP_UNTESTED;
     if (G_UNLIKELY (desktop == SOKOKE_DESKTOP_UNTESTED))
     {
@@ -300,6 +227,9 @@ sokoke_get_desktop (void)
     }
 
     return desktop;
+    #else
+    return SOKOKE_DESKTOP_UNKNOWN;
+    #endif
 }
 
 GtkWidget*
@@ -351,7 +281,7 @@ sokoke_superuser_warning_new (void)
         gtk_widget_modify_fg (GTK_WIDGET (label), GTK_STATE_NORMAL,
             &GTK_WIDGET (label)->style->fg[GTK_STATE_SELECTED]);
         gtk_widget_show (label);
-        gtk_container_add (GTK_CONTAINER(hbox), GTK_WIDGET (label));
+        gtk_container_add (GTK_CONTAINER (hbox), GTK_WIDGET (label));
         gtk_widget_show (hbox);
         return hbox;
     }
@@ -400,7 +330,7 @@ sokoke_on_entry_focus_in_event (GtkEntry*      entry,
     if (has_default)
     {
         gtk_entry_set_text (entry, "");
-        g_object_set_data (G_OBJECT(entry), "sokoke_has_default",
+        g_object_set_data (G_OBJECT (entry), "sokoke_has_default",
                            GINT_TO_POINTER (0));
         sokoke_widget_set_pango_font_style (GTK_WIDGET (entry),
                                             PANGO_STYLE_NORMAL);
@@ -419,7 +349,7 @@ sokoke_on_entry_focus_out_event (GtkEntry*      entry,
         const gchar* default_text = (const gchar*)g_object_get_data (
             G_OBJECT (entry), "sokoke_default_text");
         gtk_entry_set_text (entry, default_text);
-        g_object_set_data (G_OBJECT(entry),
+        g_object_set_data (G_OBJECT (entry),
                            "sokoke_has_default", GINT_TO_POINTER (1));
         sokoke_widget_set_pango_font_style (GTK_WIDGET (entry),
                                             PANGO_STYLE_ITALIC);
@@ -614,21 +544,79 @@ sokoke_object_get_boolean (gpointer     object,
  * gtk_action_create_menu_item(), but it won't
  * display an accelerator.
  *
+ * Note: This menu item is not a proxy and will
+ *       not reflect any changes to the action.
+ *
  * Return value: a new #GtkMenuItem
  **/
 GtkWidget*
 sokoke_action_create_popup_menu_item (GtkAction* action)
 {
     GtkWidget* menuitem;
+    GtkWidget* icon;
+    gchar* label;
+    gchar* stock_id;
+    gchar* icon_name;
+    gboolean sensitive;
+    gboolean visible;
 
     g_return_val_if_fail (GTK_IS_ACTION (action), NULL);
 
-    menuitem = gtk_image_menu_item_new ();
-    gtk_action_connect_proxy (action, menuitem);
+    g_object_get (action,
+                  "label", &label,
+                  "stock-id", &stock_id,
+                  "icon-name", &icon_name,
+                  "sensitive", &sensitive,
+                  "visible", &visible,
+                  NULL);
+    if (GTK_IS_TOGGLE_ACTION (action))
+    {
+        menuitem = gtk_check_menu_item_new_with_mnemonic (label);
+        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem),
+            gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
+    }
+    else if (stock_id)
+    {
+        if (label)
+        {
+            menuitem = gtk_image_menu_item_new_with_mnemonic (label);
+            icon = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_MENU);
+            gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), icon);
+        }
+        else
+            menuitem = gtk_image_menu_item_new_from_stock (stock_id, NULL);
+    }
+    else
+    {
+        menuitem = gtk_image_menu_item_new_with_mnemonic (label);
+        if (icon_name)
+        {
+            icon = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
+            gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), icon);
+        }
+    }
+    gtk_widget_set_sensitive (menuitem, sensitive);
+    sokoke_widget_set_visible (menuitem, visible);
+    g_signal_connect_swapped (menuitem, "activate",
+                              G_CALLBACK (gtk_action_activate), action);
 
     return menuitem;
 }
 
+/**
+ * sokoke_image_menu_item_new_ellipsized:
+ * @label: the text of the menu item
+ *
+ * Creates a new #GtkImageMenuItem containing an ellipsized label.
+ *
+ * Return value: a new #GtkImageMenuItem
+ **/
+GtkWidget*
+sokoke_image_menu_item_new_ellipsized (const gchar* label)
+{
+    return katze_image_menu_item_new_ellipsized (label);
+}
+
 /**
  * sokoke_tree_view_get_selected_iter:
  * @tree_view: a #GtkTreeView
@@ -661,3 +649,33 @@ sokoke_tree_view_get_selected_iter (GtkTreeView*   tree_view,
             return TRUE;
     return FALSE;
 }
+
+/**
+ * sokoke_same_day:
+ * @day1: a time_t timestamp value
+ * @day2: a time_t timestamp value
+ *
+ * Compares two timestamps to see if their values are on the
+ * same day.
+ *
+ * Return value: %TRUE if the day of the timestamps match.
+ **/
+gboolean
+sokoke_same_day (const time_t* day1,
+                 const time_t* day2)
+{
+    struct tm* tm1;
+    struct tm* tm2;
+    gboolean same;
+
+    tm2 = localtime (day1);
+    tm1 = (struct tm*) g_memdup (tm2, sizeof (struct tm));
+    tm2 = localtime (day2);
+
+    same = (tm1->tm_year == tm2->tm_year &&
+            tm1->tm_mon == tm2->tm_mon &&
+            tm1->tm_mday == tm2->tm_mday) ? TRUE : FALSE;
+    g_free (tm1);
+    return same;
+}
+
