@@ -131,9 +131,16 @@ sokoke_object_get_boolean            (gpointer       object,
 GtkWidget*
 sokoke_action_create_popup_menu_item (GtkAction*     action);
 
+GtkWidget*
+sokoke_image_menu_item_new_ellipsized (const gchar*  label);
+
 gboolean
 sokoke_tree_view_get_selected_iter   (GtkTreeView*   tree_view,
                                       GtkTreeModel** model,
                                       GtkTreeIter*   iter);
 
+gboolean
+sokoke_same_day                      (const time_t*  day1,
+                                      const time_t*  day2);
+
 #endif /* !__SOKOKE_H__ */
